package lsg.characters;

import lsg.helper.Dice;
import lsg.weapons.Weapon;

public class Character {

	private static String MSG_ALIVE = "(ALIVE)" ;
	private static String MSG_DEAD = "(DEAD)" ;
	
	private String name ; // Nom du personnage
	
	private int maxLife, life ; 		// Nombre de points de vie restants
	private int maxStamina, stamina ;	// Nombre de points d'action restants
	
	private Weapon weapon ;
	
	private Dice dice101 = new Dice(101) ;
	
	public Character(String name) {
		this.name = name ;
		
	}
	
	public String getName() {
		return name;
	}
	
	public void setName(String name) {
		this.name = name;
	}
	
	public int getMaxLife() {
		return maxLife;
	}
	
	protected void setMaxLife(int maxLife) {
		this.maxLife = maxLife;
	}
	
	public int getLife() {
		return life;
	}
	
	protected void setLife(int life) {
		this.life = life;
	}
	
	public int getMaxStamina() {
		return maxStamina;
	}
	
	protected void setMaxStamina(int maxStamina) {
		this.maxStamina = maxStamina;
	}
	
	public int getStamina() {
		return stamina;
	}
	
	protected void setStamina(int stamina) {
		this.stamina = stamina;
	}
	
	public boolean isAlive(){
		return life > 0 ;
	}
	
	public void printStats(){
		System.out.println(this);
	}
	
	@Override
	public String toString() {
		
		String classe = getClass().getSimpleName() ;
		String life = String.format("%5d", getLife()) ; 
		String stam = String.format("%5d", getStamina()) ; 
		
		String msg = String.format("%-20s %-20s LIFE:%-10s STAMINA:%-10s", "[ " + classe + " ]", getName(), life, stam) ;
		
		String status ;
		if(isAlive()){
			status = MSG_ALIVE ;
		}else{
			status = MSG_DEAD ;
		}
		
		return msg + status ;
	}
	
	public int attack(){
		return attackWith(this.getWeapon()) ;
	}
	
	/**
	 * Calcule une attaque en fonction d'une arme.
	 * Le calcul dépend des statistiques de l'arme et de la stamina (restante) du personnage
	 * 
	 * @param weapon : l'arme utilisée.
	 * @return la valeur de l'attaque ; 0 si l'arme est cassée.
	 */
	private int attackWith(Weapon weapon){
		int min = weapon.getMinDamage() ;
		int max = weapon.getMaxDamage() ;
		int cost = weapon.getStamCost() ;
		
		int attack = 0 ;
		
		if(!weapon.isBroken()){
			attack = min + Math.round((max-min) * dice101.roll() / 100.f) ;
			int stam = getStamina() ;
			if(cost <= stam){ // il y a assez de stam pour lancer l'attaque
				setStamina(getStamina()-cost);
			}else{
				attack = Math.round(attack * ((float)stam / cost)) ;
				setStamina(0);
			}
			
			weapon.use();
		}
		
		return attack ;
	}
	
	public Weapon getWeapon() {
		return weapon;
	}
	
	public void setWeapon(Weapon weapon) {
		this.weapon = weapon;
	}
	
	/**
	 * Calcule le nombre de PV retires 
	 * @param value : le montant des degats reçus
	 * @return le nombre de PV effectivement retires (value si assez de vie ; le reste de la vie sinon)
	 */
	public int getHitWith(int value){
		int life = getLife() ;
		int dmg ;
		dmg = (life > value) ? value : life ; // le if en opérateur ternaire
		setLife(life-dmg);
		return dmg ;
	}
	
}
